<?php

namespace Shavshukov\EventAppBase\FileReaders;

use React\Promise\PromiseInterface;
use React\Stream\WritableStreamInterface;

interface FileReaderInterface
{
    /**
     * Read file contents, buffering, but respecting the loop
     * 
     * @param string $fileName
     * @return \React\Promise\PromiseInterface<string>
     */
    public function getContents(string $fileName): PromiseInterface;
    
    /**
     * Pipe reading of the file contents into a writable stream
     * 
     * @param \React\Stream\WritableStreamInterface $writeStream
     * @param array $options
     * @return null
     */
    public function pipeContents(string $fileName, WritableStreamInterface $writeStream, array $options = []);
    
    /**
     * Get file contents as an argument for response body
     * 
     * @param string $fileName
     * @param array $options
     * @return string|\React\Stream\ReadableStreamInterface
     */
    public function asResponseArgument(string $fileName, array $options = []);
}
