<?php

namespace Shavshukov\EventAppBase\Model;

class JsonModel extends StorableModel
{
    /**
     * Maximum depth of decoding
     *
     * @var int
     */
    protected $decodeDepth = 512;
    
    /**
     * Options for json_encode
     *
     * @var int
     */
    protected $decodeOptions = 0;
    
    /**
     * Options for json_encode
     *
     * @var int
     */
    protected $encodeOptions = 0;

    /**
     * Get JsonModel from string
     * 
     * @param string $json
     * @return \static
     */
    public static function from(string $json)
    {
        $model = new static();
        $model->source = $json;
        $model->setDataAfterSourceChanged(true);
        return $model;
    }
    
    /**
     * {@inheritDoc}
     */
    protected function decode(): array
    {
        $result = json_decode($this->source, true);
        
        if ($result === null) {
            $error = json_last_error();
            $errorMsg = json_last_error_msg();
            throw new \Exception("Failed to decode JsonModel data: [{$error}] {$errorMsg}");
        }
        
        return $result;
    }

    /**
     * {@inheritDoc}
     */
    protected function encode()
    {
        $result = json_encode($this->data, $this->encodeOptions);
        
        if ($result === false) {
            $error = json_last_error();
            $errorMsg = json_last_error_msg();
            throw new \Exception("Failed to encode JsonModel data: [{$error}] {$errorMsg}");
        }
    }
    
}