<?php

namespace Shavshukov\EventAppBase\Storage;

use Shavshukov\EventAppBase\Interfaces\StorageInterface;

abstract class Storage implements StorageInterface
{
    /**
     * Source of some type
     *
     * @var mixed
     */
    protected $source;
    
    /**
     * Whether the packed source modified
     *
     * @var bool
     */
    protected $modified = false;
    
    /**
     * Back-end storage, if any
     *
     * @var \Shavshukov\EventAppBase\Interfaces\StorageInterface|null
     */
    protected $backend;
    
    /**
     * Extend storage by another storage
     * 
     * @param \Shavshukov\EventAppBase\Interfaces\StorageInterface $storage
     * @return \static
     */
    public static function extend(StorageInterface $storage)
    {
        $newStorage = new static();
        $newStorage->setBackEnd($storage);
        $newStorage->source = $storage->content();
        return $newStorage;
    }
    
    /**
     * {@inheritDoc}
     */
    public function source()
    {
        return $this->source;
    }
    
    /**
     * {@inheritDoc}
     */
    public function setBackEnd(StorageInterface $backend)
    {
        $this->backend = $backend;
    }
    
    /**
     * {@inheritDoc}
     */
    public function load(): bool
    {
        if ($this->backend) {
            $result = $this->backend->load();
            $this->source = $this->backend->content();
            return $result;
        }
        
        return false;
    }
    
    /**
     * {@inheritDoc}
     */
    public function loadAsync(): \React\Promise\PromiseInterface
    {
        if ($this->backend) {
            return $this->backend->loadAsync()->then(function ($result) {
                $this->source = $this->backend->content();
                return $result;
            });
        }
        
        return \React\Promise\resolve(false);
    }
    
    /**
     * Save back end if modified, and return whether it was saved
     * 
     * @return boolean
     */
    protected function saveBackEnd()
    {
        if ($this->backend) {
            $this->backend->modifyBy($this);
            return true;
        }
        
        return false;
    }
    
}
